@@ -52,7 +52,6 @@
 
 #ifndef ENABLE_GVFS_METADATA
 #include "gedit-metadata-manager.h"
-#define METADATA_FILE "gedit-metadata.xml"
 #endif
 
 #define GEDIT_PAGE_SETUP_FILE		"gedit-page-setup"
@@ -79,22 +78,16 @@ struct _GeditAppPrivate
 	GSettings         *window_settings;
 
 	GMenuModel        *window_menu;
+	GMenuModel        *notebook_menu;
+	GMenuModel        *tab_width_menu;
 
 	PeasExtensionSet  *extensions;
 	GNetworkMonitor   *monitor;
 };
 
-static gboolean help = FALSE;
-static gboolean version = FALSE;
-static gboolean list_encodings = FALSE;
-static gchar *encoding_charset = NULL;
 static gboolean new_window = FALSE;
 static gboolean new_document = FALSE;
 static gchar *geometry = NULL;
-static gboolean wait = FALSE;
-static gboolean gapplication_service = FALSE;
-static gboolean standalone = FALSE;
-static gchar **remaining_args = NULL;
 static const GeditEncoding *encoding = NULL;
 static GInputStream *stdin_stream = NULL;
 static GSList *file_list = NULL;
@@ -104,86 +97,64 @@ static GApplicationCommandLine *command_line = NULL;
 
 static const GOptionEntry options[] =
 {
-	/* Help */
-	{
-		"help", '?', 0, G_OPTION_ARG_NONE, &help,
-		N_("Show the application's help"), NULL
-	},
-
 	/* Version */
 	{
-		"version", 'V', 0, G_OPTION_ARG_NONE, &version,
+		"version", 'V', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Show the application's version"), NULL
 	},
 
 	/* List available encodings */
 	{
-		"list-encodings", '\0', 0, G_OPTION_ARG_NONE, &list_encodings,
+		"list-encodings", '\0', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Display list of possible values for the encoding option"),
 		NULL
 	},
 
 	/* Encoding */
 	{
-		"encoding", '\0', 0, G_OPTION_ARG_STRING,
-		&encoding_charset,
+		"encoding", '\0', 0, G_OPTION_ARG_STRING, NULL,
 		N_("Set the character encoding to be used to open the files listed on the command line"),
 		N_("ENCODING")
 	},
 
 	/* Open a new window */
 	{
-		"new-window", '\0', 0, G_OPTION_ARG_NONE,
-		&new_window,
+		"new-window", '\0', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Create a new top-level window in an existing instance of gedit"),
 		NULL
 	},
 
 	/* Create a new empty document */
 	{
-		"new-document", '\0', 0, G_OPTION_ARG_NONE,
-		&new_document,
+		"new-document", '\0', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Create a new document in an existing instance of gedit"),
 		NULL
 	},
 
 	/* Window geometry */
 	{
-		"geometry", 'g', 0, G_OPTION_ARG_STRING,
-		&geometry,
+		"geometry", 'g', 0, G_OPTION_ARG_STRING, NULL,
 		N_("Set the size and position of the window (WIDTHxHEIGHT+X+Y)"),
 		N_("GEOMETRY")
 	},
 
-	/* GApplication service mode */
-	{
-		"gapplication-service", '\0', 0, G_OPTION_ARG_NONE,
-		&gapplication_service,
-		N_("Enter GApplication service mode"),
-		NULL
-	},
-
 	/* Wait for closing documents */
 	{
-		"wait", 'w', 0, G_OPTION_ARG_NONE,
-		&wait,
+		"wait", 'w', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Open files and block process until files are closed"),
 		NULL
 	},
 
 	/* New instance */
 	{
-		"standalone", 's', 0, G_OPTION_ARG_NONE,
-		&standalone,
+		"standalone", 's', 0, G_OPTION_ARG_NONE, NULL,
 		N_("Run gedit in standalone mode"),
 		NULL
 	},
 
 	/* collects file arguments */
 	{
-		G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY,
-		&remaining_args,
-		NULL,
+		G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, NULL, NULL,
 		N_("[FILE...] [+LINE[:COLUMN]]")
 	},
 
@@ -438,10 +409,6 @@ gedit_app_startup (GApplication *application)
 	GtkSourceStyleSchemeManager *manager;
 	const gchar *dir;
 	gchar *icon_dir;
-#ifndef ENABLE_GVFS_METADATA
-	const gchar *cache_dir;
-	gchar *metadata_filename;
-#endif
 	GError *error = NULL;
 	GFile *css_file;
 	GtkCssProvider *provider;
@@ -473,14 +440,7 @@ gedit_app_startup (GApplication *application)
 	g_free (icon_dir);
 
 #ifndef ENABLE_GVFS_METADATA
-	/* Setup metadata-manager */
-	cache_dir = gedit_dirs_get_user_cache_dir ();
-
-	metadata_filename = g_build_filename (cache_dir, METADATA_FILE, NULL);
-
 	gedit_metadata_manager_init (metadata_filename);
-
-	g_free (metadata_filename);
 #endif
 
 	/* Load settings */
@@ -521,6 +481,9 @@ gedit_app_startup (GApplication *application)
 		{
 			app->priv->window_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "gear_menu_noappmenu"));
 		}
+
+		app->priv->notebook_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "notebook_menu"));
+		app->priv->tab_width_menu = G_MENU_MODEL (gtk_builder_get_object (builder, "tab_width_menu"));
 	}
 
 	g_object_unref (builder);
@@ -802,68 +765,16 @@ gedit_app_activate (GApplication *application)
 	gtk_window_present (GTK_WINDOW (window));
 }
 
-static GOptionContext *
-get_option_context (void)
-{
-	GOptionContext *context;
-
-	context = g_option_context_new (_("- Edit text files"));
-	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-	g_option_context_add_group (context, gtk_get_option_group (FALSE));
-
-#ifdef ENABLE_INTROSPECTION
-	g_option_context_add_group (context, g_irepository_get_option_group ());
-#endif
-
-	return context;
-}
-
-static gboolean
-option_context_parse (GOptionContext  *context,
-                      gchar          **arguments,
-                      GError         **error)
-{
-	gint argc;
-	gchar **argv;
-	gint i;
-	gboolean ret;
-
-	/* We have to make an extra copy of the array, since g_option_context_parse()
-	 * assumes that it can remove strings from the array without freeing them.
-	 */
-	argc = g_strv_length (arguments);
-	argv = g_new (gchar *, argc);
-	for (i = 0; i < argc; i++)
-	{
-		argv[i] = arguments[i];
-	}
-
-	ret = g_option_context_parse (context, &argc, &argv, error);
-
-	g_free (argv);
-
-	return ret;
-}
-
 static void
 clear_options (void)
 {
-	g_free (encoding_charset);
-	g_strfreev (remaining_args);
 	g_free (geometry);
 	g_clear_object (&stdin_stream);
 	g_slist_free_full (file_list, g_object_unref);
 
-	help = FALSE;
-	version = FALSE;
-	list_encodings = FALSE;
-	encoding_charset = NULL;
 	new_window = FALSE;
 	new_document = FALSE;
 	geometry = NULL;
-	wait = FALSE;
-	standalone = FALSE;
-	remaining_args = NULL;
 	encoding = NULL;
 	file_list = NULL;
 	line_position = 0;
@@ -900,132 +811,89 @@ static gint
 gedit_app_command_line (GApplication            *application,
                         GApplicationCommandLine *cl)
 {
-	gchar **arguments;
-	GOptionContext *context;
-	GError *error = NULL;
+	GVariantDict *options;
+	const gchar *encoding_charset;
+	const gchar **remaining_args;
 
-	arguments = g_application_command_line_get_arguments (cl, NULL);
+	options = g_application_command_line_get_options_dict (cl);
 
-	context = get_option_context ();
+	g_variant_dict_lookup (options, "new-window", "b", &new_window);
+	g_variant_dict_lookup (options, "new-document", "b", &new_document);
+	g_variant_dict_lookup (options, "geometry", "s", &geometry);
 
-	/* Avoid exit() on the main instance */
-	g_option_context_set_help_enabled (context, FALSE);
-
-	if (!option_context_parse (context, arguments, &error))
+	if (g_variant_dict_contains (options, "wait"))
 	{
-		/* We should never get here since parsing would have
-		 * failed on the client side... */
-		g_application_command_line_printerr (cl,
-		                                     _("%s\nRun '%s --help' to see a full list of available command line options.\n"),
-		                                     error->message, arguments[0]);
-
-		g_error_free (error);
-		g_application_command_line_set_exit_status (cl, 1);
+		command_line = cl;
 	}
-	else
+
+	if (g_variant_dict_lookup (options, "encoding", "&s", &encoding_charset))
 	{
-		if (wait)
+		encoding = gedit_encoding_get_from_charset (encoding_charset);
+
+		if (encoding == NULL)
 		{
-			command_line = cl;
+			g_application_command_line_printerr (cl,
+							     _("%s: invalid encoding."),
+							     encoding_charset);
 		}
+	}
 
-		/* Parse encoding */
-		if (encoding_charset)
+	/* Parse filenames */
+	if (g_variant_dict_lookup (options, G_OPTION_REMAINING, "^a&ay", &remaining_args))
+	{
+		gint i;
+
+		for (i = 0; remaining_args[i]; i++)
 		{
-			encoding = gedit_encoding_get_from_charset (encoding_charset);
-
-			if (encoding == NULL)
+			if (*remaining_args[i] == '+')
 			{
-				g_application_command_line_printerr (cl,
-				                                     _("%s: invalid encoding."),
-				                                     encoding_charset);
-			}
-
-			g_free (encoding_charset);
-		}
-
-		/* Parse filenames */
-		if (remaining_args)
-		{
-			gint i;
-
-			for (i = 0; remaining_args[i]; i++)
-			{
-				if (*remaining_args[i] == '+')
+				if (*(remaining_args[i] + 1) == '\0')
 				{
-					if (*(remaining_args[i] + 1) == '\0')
-					{
-						/* goto the last line of the document */
-						line_position = G_MAXINT;
-						column_position = 0;
-					}
-					else
-					{
-						get_line_column_position (remaining_args[i] + 1,
-							                  &line_position,
-							                  &column_position);
-					}
-				}
-				else if (*remaining_args[i] == '-' && *(remaining_args[i] + 1) == '\0')
-				{
-					stdin_stream = g_application_command_line_get_stdin (cl);
+					/* goto the last line of the document */
+					line_position = G_MAXINT;
+					column_position = 0;
 				}
 				else
 				{
-					GFile *file;
-
-					file = g_application_command_line_create_file_for_arg (cl, remaining_args[i]);
-					file_list = g_slist_prepend (file_list, file);
+					get_line_column_position (remaining_args[i] + 1,
+								  &line_position,
+								  &column_position);
 				}
 			}
+			else if (*remaining_args[i] == '-' && *(remaining_args[i] + 1) == '\0')
+			{
+				stdin_stream = g_application_command_line_get_stdin (cl);
+			}
+			else
+			{
+				GFile *file;
 
-			file_list = g_slist_reverse (file_list);
+				file = g_application_command_line_create_file_for_arg (cl, remaining_args[i]);
+				file_list = g_slist_prepend (file_list, file);
+			}
 		}
 
-		g_application_activate (application);
+		file_list = g_slist_reverse (file_list);
+		g_free (remaining_args);
 	}
 
-	g_option_context_free (context);
-	g_strfreev (arguments);
+	g_application_activate (application);
 	clear_options ();
 
 	return 0;
 }
 
-static gboolean
-gedit_app_local_command_line (GApplication   *application,
-                              gchar        ***arguments,
-                              gint           *exit_status)
+static gint
+gedit_app_handle_local_options (GApplication *application,
+                                GVariantDict *options)
 {
-	GOptionContext *context;
-	GError *error = NULL;
-	gboolean ret = FALSE;
-
-	/* Handle some of the option without contacting the main instance */
-	context = get_option_context ();
-
-	if (!option_context_parse (context, *arguments, &error))
-	{
-		g_printerr (_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
-		           error->message, (*arguments)[0]);
-
-		g_error_free (error);
-		*exit_status = 1;
-		ret = TRUE;
-	}
-	else if (gapplication_service)
-	{
-		GApplicationFlags old_flags;
-
-		old_flags = g_application_get_flags (application);
-		g_application_set_flags (application, old_flags | G_APPLICATION_IS_SERVICE);
-	}
-	else if (version)
+	if (g_variant_dict_contains (options, "version"))
 	{
 		g_print ("%s - Version %s\n", g_get_application_name (), VERSION);
-		ret = TRUE;
+		return 0;
 	}
-	else if (list_encodings)
+
+	if (g_variant_dict_contains (options, "list-encodings"))
 	{
 		gint i = 0;
 		const GeditEncoding *enc;
@@ -1037,9 +905,10 @@ gedit_app_local_command_line (GApplication   *application,
 			++i;
 		}
 
-		ret = TRUE;
+		return 0;
 	}
-	else if (standalone)
+
+	if (g_variant_dict_contains (options, "standalone"))
 	{
 		GApplicationFlags old_flags;
 
@@ -1047,10 +916,15 @@ gedit_app_local_command_line (GApplication   *application,
 		g_application_set_flags (application, old_flags | G_APPLICATION_NON_UNIQUE);
 	}
 
-	g_option_context_free (context);
-	clear_options ();
+	if (g_variant_dict_contains (options, "wait"))
+	{
+		GApplicationFlags old_flags;
 
-	return ret ? ret : G_APPLICATION_CLASS (gedit_app_parent_class)->local_command_line (application, arguments, exit_status);
+		old_flags = g_application_get_flags (application);
+		g_application_set_flags (application, old_flags | G_APPLICATION_IS_LAUNCHER);
+	}
+
+	return -1;
 }
 
 static gboolean
@@ -1271,7 +1145,7 @@ gedit_app_class_init (GeditAppClass *klass)
 	app_class->startup = gedit_app_startup;
 	app_class->activate = gedit_app_activate;
 	app_class->command_line = gedit_app_command_line;
-	app_class->local_command_line = gedit_app_local_command_line;
+	app_class->handle_local_options = gedit_app_handle_local_options;
 	app_class->shutdown = gedit_app_shutdown;
 
 	klass->show_help = gedit_app_show_help_impl;
@@ -1398,6 +1272,12 @@ gedit_app_init (GeditApp *app)
 			  "network-changed",
 			  G_CALLBACK (get_network_available),
 			  app);
+
+	g_application_add_main_option_entries (G_APPLICATION (app), options);
+
+#ifdef ENABLE_INTROSPECTION
+	g_application_add_option_group (G_APPLICATION (app), g_irepository_get_option_group ());
+#endif
 }
 
 /* Generates a unique string for a window role */
@@ -1477,6 +1357,37 @@ gedit_app_create_window (GeditApp  *app,
 	return window;
 }
 
+/**
+ * gedit_app_get_main_windows:
+ * @app: the #GeditApp
+ *
+ * Returns all #GeditWindows currently open in #GeditApp.
+ * This differs from gtk_application_get_windows() since it does not
+ * include the preferences dialog and other auxiliary windows.
+ *
+ * Return value: (element-type Gedit.Window) (transfer container):
+ * a newly allocated list of #GeditWindow objects
+ */
+GList *
+gedit_app_get_main_windows (GeditApp *app)
+{
+	GList *res = NULL;
+	GList *windows, *l;
+
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	for (l = windows; l != NULL; l = g_list_next (l))
+	{
+		if (GEDIT_IS_WINDOW (l->data))
+		{
+			res = g_list_prepend (res, l->data);
+		}
+	}
+
+	return g_list_reverse (res);
+}
+
 /**
  * gedit_app_get_documents:
  * @app: the #GeditApp
@@ -1592,7 +1503,6 @@ gedit_app_process_window_event (GeditApp    *app,
     return FALSE;
 }
 
-
 static GMenuModel *
 find_extension_point_section (GMenuModel  *model,
                               const gchar *extension_point)
@@ -1626,8 +1536,13 @@ app_lockdown_changed (GeditApp *app)
 	windows = gtk_application_get_windows (GTK_APPLICATION (app));
 	for (l = windows; l != NULL; l = g_list_next (l))
 	{
-		_gedit_window_set_lockdown (GEDIT_WINDOW (l->data),
-		                            app->priv->lockdown);
+		GtkWindow *window = l->data;
+
+		if (GEDIT_IS_WINDOW (window))
+		{
+			_gedit_window_set_lockdown (GEDIT_WINDOW (window),
+				                    app->priv->lockdown);
+		}
 	}
 
 	g_object_notify (G_OBJECT (app), "lockdown");
@@ -1725,6 +1640,22 @@ _gedit_app_get_window_menu (GeditApp *app)
 	return app->priv->window_menu;
 }
 
+GMenuModel *
+_gedit_app_get_notebook_menu (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	return app->priv->notebook_menu;
+}
+
+GMenuModel *
+_gedit_app_get_tab_width_menu (GeditApp *app)
+{
+	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
+
+	return app->priv->tab_width_menu;
+}
+
 GeditMenuExtension *
 _gedit_app_extend_menu (GeditApp    *app,
                        const gchar *extension_point)
