@@ -15,15 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
- */
-
-/*
- * Modified by the gedit Team, 2005. See the AUTHORS file for a
- * list of people on the gedit Team.
- * See the ChangeLog files for a list of changes.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -778,7 +770,7 @@ show_loading_info_bar (GeditTab *tab)
 					       name_markup);
 		}
 
-		bar = gedit_progress_info_bar_new (GTK_STOCK_REVERT_TO_SAVED,
+		bar = gedit_progress_info_bar_new ("document-revert",
 						   msg,
 						   TRUE);
 	}
@@ -801,7 +793,7 @@ show_loading_info_bar (GeditTab *tab)
 					       name_markup);
 		}
 
-		bar = gedit_progress_info_bar_new (GTK_STOCK_OPEN,
+		bar = gedit_progress_info_bar_new ("document-open",
 						   msg,
 						   TRUE);
 	}
@@ -886,7 +878,7 @@ show_saving_info_bar (GeditTab *tab)
 		msg = g_strdup_printf (_("Saving %s"), from_markup);
 	}
 
-	bar = gedit_progress_info_bar_new (GTK_STOCK_SAVE,
+	bar = gedit_progress_info_bar_new ("document-save",
 					   msg,
 					   FALSE);
 
@@ -1639,6 +1631,52 @@ on_drop_uris (GeditView *view,
 	g_signal_emit (G_OBJECT (tab), signals[DROP_URIS], 0, uri_list);
 }
 
+static void
+network_available_warning_info_bar_response (GtkWidget *info_bar,
+					     gint      response_id,
+					     GeditTab  *tab)
+{
+	if (response_id == GTK_RESPONSE_CLOSE)
+	{
+		gtk_widget_hide (info_bar);
+	}
+}
+
+void
+_gedit_tab_set_network_available (GeditTab *tab,
+			          gboolean enable)
+{
+	GeditDocument *doc;
+	GFile *location;
+
+	g_return_if_fail (GEDIT_IS_TAB (tab));
+
+	doc = gedit_tab_get_document (tab);
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+	location = gedit_document_get_location (doc);
+
+	if (!gedit_document_is_local (doc))
+	{
+		if (enable)
+		{
+			set_info_bar (tab, NULL);
+		}
+		else
+		{
+			GtkWidget *bar = gedit_network_unavailable_info_bar_new (location);
+			gtk_info_bar_set_default_response (GTK_INFO_BAR (bar),
+							   GTK_RESPONSE_CLOSE);
+			g_signal_connect (bar,
+					  "response",
+					  G_CALLBACK (network_available_warning_info_bar_response),
+					  tab);
+
+			set_info_bar (tab, bar);
+		}
+	}
+}
+
 static void
 gedit_tab_init (GeditTab *tab)
 {
@@ -2590,22 +2628,13 @@ print_cancelled (GtkWidget        *bar,
 }
 
 static void
-show_printing_info_bar (GeditTab *tab, gboolean preview)
+show_printing_info_bar (GeditTab *tab)
 {
 	GtkWidget *bar;
 
-	if (preview)
-	{
-		bar = gedit_progress_info_bar_new (GTK_STOCK_PRINT_PREVIEW,
-							"",
-							TRUE);
-	}
-	else
-	{
-		bar = gedit_progress_info_bar_new (GTK_STOCK_PRINT,
-							"",
-							TRUE);
-	}
+	bar = gedit_progress_info_bar_new ("document-print",
+					   "",
+					   TRUE);
 
 	g_signal_connect (bar,
 			  "response",
@@ -2620,7 +2649,6 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 				  GtkPrintOperationAction  print_action)
 {
 	GeditView *view;
-	gboolean is_preview;
 	GtkPageSetup *setup;
 	GtkPrintSettings *settings;
 	GtkPrintOperationResult res;
@@ -2631,13 +2659,12 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 
 	view = gedit_tab_get_view (tab);
 
-	is_preview = (print_action == GTK_PRINT_OPERATION_ACTION_PREVIEW);
 
 	tab->priv->print_job = gedit_print_job_new (view);
 	g_object_add_weak_pointer (G_OBJECT (tab->priv->print_job),
 				   (gpointer *) &tab->priv->print_job);
 
-	show_printing_info_bar (tab, is_preview);
+	show_printing_info_bar (tab);
 
 	g_signal_connect (tab->priv->print_job,
 			  "printing",
@@ -2652,10 +2679,14 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 			  G_CALLBACK (done_printing_cb),
 			  tab);
 
-	if (is_preview)
+	if (print_action == GTK_PRINT_OPERATION_ACTION_PREVIEW)
+	{
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINT_PREVIEWING);
+	}
 	else
+	{
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_PRINTING);
+	}
 
 	setup = get_page_setup (tab);
 	settings = get_print_settings (tab);
