@@ -15,24 +15,13 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
- */
-
-/*
- * Modified by the gedit Team, 2005. See the AUTHORS file for a
- * list of people on the gedit Team.
- * See the ChangeLog files for a list of changes.
- *
- * $Id$
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GEDIT_TAB_H__
 #define __GEDIT_TAB_H__
 
 #include <gtk/gtk.h>
-
 #include <gedit/gedit-view.h>
 #include <gedit/gedit-document.h>
 
@@ -177,6 +166,10 @@ gboolean	 _gedit_tab_get_can_close	(GeditTab	     *tab);
 
 GtkWidget	*_gedit_tab_get_view_frame	(GeditTab            *tab);
 
+void		 _gedit_tab_set_network_available
+						(GeditTab	     *tab,
+						 gboolean	     enable);
+
 G_END_DECLS
 
 #endif  /* __GEDIT_TAB_H__  */
