@@ -56,22 +56,21 @@ struct _GeditWindowPrivate
 
 	/* Widgets for fullscreen mode */
 	GtkWidget      *fullscreen_controls;
+	GtkWidget      *fullscreen_eventbox;
+	GtkWidget      *fullscreen_headerbar;
 	GtkWidget      *fullscreen_open_menu;
 	GtkMenuButton  *fullscreen_gear_button;
 	gboolean        fullscreen_controls_setup;
-	guint           fullscreen_animation_timeout_id;
-	gboolean        fullscreen_animation_enter;
 
 	/* statusbar and context ids for statusbar messages */
 	GtkWidget      *statusbar;
 	GtkWidget      *tab_width_combo;
-	GtkWidget      *tab_width_combo_menu;
 	GtkWidget      *language_button;
 	GtkWidget      *language_button_label;
+	GtkWidget      *language_popover;
 	guint           generic_message_cid;
 	guint           tip_message_cid;
 	guint 	        bracket_match_message_cid;
-	GBinding       *spaces_instead_of_tabs_binding;
 	guint 	        tab_width_id;
 	guint 	        language_changed_id;
 
