@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -129,25 +128,20 @@ G_DEFINE_DYNAMIC_TYPE_EXTENDED (GeditFileBrowserPlugin,
 )
 
 static GSettings *
-settings_try_new (const gchar *schema)
+settings_try_new (const gchar *schema_id)
 {
-	const gchar * const * schemas;
 	GSettings *settings = NULL;
+	GSettingsSchemaSource *source;
+	GSettingsSchema *schema;
 
-	schemas = g_settings_list_schemas ();
+	source = g_settings_schema_source_get_default ();
 
-	if (schemas == NULL)
+	schema = g_settings_schema_source_lookup (source, schema_id, TRUE);
+
+	if (schema != NULL)
 	{
-		return NULL;
-	}
-
-	for (; *schemas != NULL; schemas++)
-	{
-		if (g_strcmp0 (*schemas, schema) == 0)
-		{
-			settings = g_settings_new (schema);
-			break;
-		}
+		settings = g_settings_new_full (schema, NULL, NULL);
+		g_settings_schema_unref (schema);
 	}
 
 	return settings;
@@ -486,8 +480,7 @@ gedit_file_browser_plugin_activate (GeditWindowActivatable *activatable)
 {
 	GeditFileBrowserPlugin *plugin = GEDIT_FILE_BROWSER_PLUGIN (activatable);
 	GeditFileBrowserPluginPrivate *priv;
-	GeditPanel *panel;
-	GtkWidget *image;
+	GtkWidget *panel;
 	GeditFileBrowserStore *store;
 
 	priv = plugin->priv;
@@ -529,14 +522,10 @@ gedit_file_browser_plugin_activate (GeditWindowActivatable *activatable)
 
 	panel = gedit_window_get_side_panel (priv->window);
 
-	image = gtk_image_new_from_icon_name ("system-file-manager-symbolic",
-	                                      GTK_ICON_SIZE_MENU);
-	gtk_widget_show (image);
-	gedit_panel_add_item (panel,
+	gtk_stack_add_titled (GTK_STACK (panel),
 	                      GTK_WIDGET (priv->tree_widget),
 	                      "GeditFileBrowserPanel",
-	                      _("File Browser"),
-	                      image);
+	                      _("File Browser"));
 
 	gtk_widget_show (GTK_WIDGET (priv->tree_widget));
 
@@ -589,7 +578,7 @@ gedit_file_browser_plugin_deactivate (GeditWindowActivatable *activatable)
 {
 	GeditFileBrowserPlugin *plugin = GEDIT_FILE_BROWSER_PLUGIN (activatable);
 	GeditFileBrowserPluginPrivate *priv = plugin->priv;
-	GeditPanel *panel;
+	GtkWidget *panel;
 
 
 	/* Unregister messages from the bus */
@@ -619,7 +608,7 @@ gedit_file_browser_plugin_deactivate (GeditWindowActivatable *activatable)
 	}
 
 	panel = gedit_window_get_side_panel (priv->window);
-	gedit_panel_remove_item (panel, GTK_WIDGET (priv->tree_widget));
+	gtk_container_remove (GTK_CONTAINER (panel), GTK_WIDGET (priv->tree_widget));
 }
 
 static void
@@ -930,8 +919,7 @@ on_confirm_no_trash_cb (GeditFileBrowserWidget *widget,
 	                                                       GTK_MESSAGE_QUESTION,
 	                                                       message,
 	                                                       secondary,
-	                                                       GTK_STOCK_DELETE,
-	                                                       NULL);
+	                                                       _("_Delete"));
 	g_free (secondary);
 
 	return result;
@@ -969,8 +957,7 @@ on_confirm_delete_cb (GeditFileBrowserWidget *widget,
 	                                                       GTK_MESSAGE_QUESTION,
 	                                                       message,
 	                                                       secondary,
-	                                                       GTK_STOCK_DELETE,
-	                                                       NULL);
+	                                                       _("_Delete"));
 
 	g_free (message);
 
