@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef __GEDIT_FILE_BROWSER_UTILS_H__
@@ -40,7 +39,6 @@ gboolean	 gedit_file_browser_utils_confirmation_dialog	(GeditWindow    *window,
 								 GtkMessageType  type,
 								 gchar const    *message,
 								 gchar const    *secondary,
-								 gchar const    *button_stock,
 								 gchar const    *button_label);
 
 #endif /* __GEDIT_FILE_BROWSER_UTILS_H__ */
