@@ -15,8 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -121,7 +120,9 @@ struct _GeditFileBrowserWidgetPrivate
 	GMenuModel *bookmarks_menu;
 
 	GtkWidget *previous_button;
+	GtkWidget *previous_image;
 	GtkWidget *next_button;
+	GtkWidget *next_image;
 
 	GtkWidget *combo;
 	GtkTreeStore *combo_model;
@@ -557,7 +558,9 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass *klass)
 	gtk_widget_class_set_template_from_resource (widget_class,
 	                                             "/org/gnome/gedit/plugins/file-browser/ui/gedit-file-browser-widget.ui");
 	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, previous_button);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, previous_image);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, next_button);
+	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, next_image);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, combo);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, combo_model);
 	gtk_widget_class_bind_template_child_private (widget_class, GeditFileBrowserWidget, location_entry);
@@ -949,10 +952,13 @@ gedit_file_browser_widget_init (GeditFileBrowserWidget *obj)
 {
 	GtkBuilder *builder;
 	GAction *action;
+	gboolean rtl;
 	GError *error = NULL;
 
 	obj->priv = gedit_file_browser_widget_get_instance_private (obj);
 
+	rtl = gtk_widget_get_direction (GTK_WIDGET (obj)) == GTK_TEXT_DIR_RTL;
+
 	obj->priv->filter_pattern_str = g_strdup ("");
 	obj->priv->bookmarks_hash = g_hash_table_new_full (g_file_hash,
 			                                   (GEqualFunc)g_file_equal,
@@ -1000,6 +1006,13 @@ gedit_file_browser_widget_init (GeditFileBrowserWidget *obj)
 
 	gtk_widget_init_template (GTK_WIDGET (obj));
 
+	gtk_image_set_from_icon_name (GTK_IMAGE (obj->priv->previous_image),
+				      rtl ? "go-previous-rtl-symbolic" : "go-previous-symbolic",
+				      GTK_ICON_SIZE_MENU);
+	gtk_image_set_from_icon_name (GTK_IMAGE (obj->priv->next_image),
+				      rtl ? "go-next-rtl-symbolic" : "go-next-symbolic",
+				      GTK_ICON_SIZE_MENU);
+
 	g_signal_connect (obj->priv->previous_button, "button-press-event",
 	                  G_CALLBACK (on_location_button_press_event), obj);
 	g_signal_connect (obj->priv->next_button, "button-press-event",
